

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/*                                                          */
/*    Author:   Thomas Pantzer                              */
/*    Date:     Tue Nov 28 20:15:36 MEZ 1995                */
/*    File:     eco_serv.cc                                 */
/*    Function:	Main-Module Echo-Server 	            */	
/*             						    */
/*__________________________________________________________*/






#define MAIN_G

#include "utils.h"
#include "netutils.h"
#include "grabber.h"

#include <unistd.h>
#include <fcntl.h>
#include <string.h>

#ifndef MAX_PEERS
#define MAX_PEERS 4
#endif

#ifndef BLOCKED_TIMEOUT
#define BLOCKED_TIMEOUT	(60*5)		// 8 minutes measured in seconds
#endif

#ifndef WEBCAM_SERV			// the service name as string 
#define WEBCAM_SERV "webcamd"
#endif



#ifdef DEBUG
#define CHECK_POINT(x,y) fprintf(stderr,"%s: %d\n",x,y);
#else
#define CHECK_POINT(x,y)
#endif

//#define FRAG_SIZE (PIC_SIZE/64)
//#define FRAG_SIZE (WIDTH*9) // largest fragment below standard MTU size
//#define FRAG_SIZE (WIDTH*112)
//#define FRAG_SIZE (PIC_SIZE)

#define FRAG_SIZE (PIC_SIZE+5)

typedef struct {
        	int Socket;
                char connected,listen;
                char Name[256];
                char *Buffer;
                int sent;
                time_t stime,timeout;
               } t_peer; 

typedef struct {
                int ID;
                char str[128];
   	       } t_cmd;
               
typedef struct {
                int red,green,blue;
               } t_color; 

               
  t_cmd Cmd[] = {
                 {CMD_GET,"get"},
                 {CMD_RESEND,"resend"}, 
                 {CMD_CONTRAST,"contrast"},
                 {CMD_BRITE,"brite"},  
                 {CMD_SYNC,"sync"}, 
                 {CMD_SOURCE,"source"},
                 {CMD_HUE,"hue"},
                 {CMD_COLOR,"color"},   
                 {CMD_FREQUENCY,"frequency"}, 
                 {0,"null cmd"}
                };             
        	
  t_color col[256];

  int Accept_Sock;
  int peer_cnt;
  t_peer Peer[MAX_PEERS]; 
  struct sockaddr_in ClientAddr;
  struct fd_set Read_FD_Set;
  int FD_MaxNum;




void build_fdset(void)
{
  struct timeval TimeOut;

//Wartezeit auf einen Event
    TimeOut.tv_sec=0;
    TimeOut.tv_usec=10000;

//alle aktiven File-Descriptoren sollen von "select()" geprft werden
    for (int i=0;i<MAX_PEERS;i++)
      if (Peer[i].connected == true)
         FD_SET(Peer[i].Socket,&Read_FD_Set);
}



void accept_new(void)
{  
   int CNameLen;
   time_t TM;
   char st[256];
   char *s;

   if (peer_cnt >= MAX_PEERS)
      return;

   if (FD_Ready(Accept_Sock) == false)
      {
//       CHECK_POINT("cancel accept",Accept_Sock);
       return;
      } 
   
   CNameLen = sizeof(ClientAddr);
   Peer[peer_cnt].Socket = accept(Accept_Sock,(struct sockaddr *)&ClientAddr,&CNameLen);
            
   if (Peer[peer_cnt].Socket == -1)
      ErrorMsg("accept()",errno);
    else 
      {
       Peer[peer_cnt].connected = true;
       Peer[peer_cnt].listen = true;
       Peer[peer_cnt].Buffer = (char *)malloc(PIC_SIZE + 10);
       
       fcntl(Peer[peer_cnt].Socket,F_SETFL,O_NONBLOCK);
       
       hostname_str(&ClientAddr,Peer[peer_cnt].Name);
       
       Peer[peer_cnt].stime = time(&TM);
       sprintf(st,"%s",ctime(&TM));
       s = strchr(st,'\n');

       if (s != NULL)
          *s = '\0';
       
       fprintf(stdout,"%d. connect \aat %s: %s\n",
                peer_cnt+1,st,Peer[peer_cnt].Name);

       FD_MaxNum = max(FD_MaxNum,Peer[peer_cnt].Socket);
       peer_cnt++;
      }   
      
   CHECK_POINT("finish accept",peer_cnt);   
}   


char error_handling(t_peer *P, time_t etime)
{
      switch(errno)
        {
         case 0: 
         case 2:
         case 9:    // bad file number
         case 14:   // bad address
         case 32:   // brocken pipe
         case 107:
           return false;
           break;
                    
         default:
           ErrorMsg("read()",errno);

         case 11: // EWOULDBLOCK
           if ((etime - P->timeout) > BLOCKED_TIMEOUT)
              {
               fprintf(stderr,"EWOULDBLOCK timeout\n");
               return false;
              } 
    
           break;    
        } // switch(errno)
        
   return true;     
}



char process_input(t_peer *P)
{
  char message[256];

  if (P->connected == false)
     return false;   // is invalid

  if (FD_Ready(P->Socket) == false)
     return true;    // might not be invalid
     
  int rd = read(P->Socket,&message[0],128);
           
  if (rd < 1)
     {// error
      if (error_handling(P,time(0)) == false)
         return false;
     } // if (rd <= 0)
    else
      {
       P->timeout = time(0);
       
       message[rd] = '\0';
       
       int i,j,k;
       i = 0;
       while (Cmd[i].ID != 0)
         {
          j = strlen(Cmd[i].str);
          if (strncmp(message,Cmd[i].str,j) == 0)
             {
              char sfmt[128];
              sprintf(sfmt,"%s %%d",Cmd[i].str);
              
              if (sscanf(message,sfmt,&k) == 1)
                 {
                  
                  switch (Cmd[i].ID)
                    {
                     case CMD_SYNC:
//                       Peer[i].sent = 0;
// 			 P->listen = true;
                       break;
                       
                     case CMD_COLOR:
                     case CMD_HUE:
                     case CMD_SOURCE:
                     case CMD_CONTRAST:
                     case CMD_BRITE:
                     case CMD_FREQUENCY:
                       SetCtrl(Cmd[i].ID, k);
                       break;
                       
                     default:
                       CHECK_POINT("got unknown command",i);
                       CHECK_POINT("with argument",k);
                       break; 
                    } // switch command code
                    
                 } // if argument scanned
                 
             } // if prefix equal
     
           i++;             
         } // while not all commands tested
               
       P->listen = true;
      }  // if some bytes read
      
  return true;    
}


char process_output(t_peer *P)
{
       int sz,wr;
       
//       if (Peer[i].listen == true)   
//          {// peer is waiting for a fresh pic

    if (P->sent == 0) 
       { // fill the buffer with the fresh pic
        strcpy(P->Buffer,"start");
                               
        for (int j=0; j<PIC_SIZE; j++)
          P->Buffer[j+5] = picture[j];
       }   
 
    sz = PIC_SIZE - P->sent;  // determine bytes to send
           
    if (sz > FRAG_SIZE)
       sz = FRAG_SIZE;
       
    if (sz <= 0)
       {
        P->sent = 0;
        return true;   
       } 

    wr = write(P->Socket,&P->Buffer[P->sent],sz);

    if (wr > 0)
       { 
        P->sent += wr;

        if (P->sent == PIC_SIZE)
           P->listen = false;

        // reset timeout counter because some bytes were written
        P->timeout = time(0);
       }
     else
       {// error
        if (error_handling(P,time(0)) == false)
           return false;
           
       } // if (wr <= 0)
       
   return true;    
}


void delete_peer(t_peer *P)
{
   int sp = P->Socket;
   
   ShutDown(P->Socket);

   if ((P->Socket == sp) && (sp > 0))
      close(P->Socket);
  
   long sec;
   
   sec = time(0) - P->stime;
   fprintf(stdout,"client %s disconnected after %ld sec.\n",P->Name,sec);       
                
   peer_cnt--;
   P->connected = Peer[peer_cnt].connected;
   P->listen = Peer[peer_cnt].listen;
   P->Socket = Peer[peer_cnt].Socket;
   P->sent = Peer[peer_cnt].sent;
   P->stime = Peer[peer_cnt].stime;
   P->timeout = Peer[peer_cnt].timeout;
   
   free(P->Buffer);
   P->Buffer = NULL;
   
   P->Buffer = Peer[peer_cnt].Buffer;
   
   strcpy(P->Name,Peer[peer_cnt].Name);
      
   Peer[peer_cnt].Socket = -1;
   Peer[peer_cnt].connected = false;
   Peer[peer_cnt].listen = false;
   Peer[peer_cnt].sent = 0;
   Peer[peer_cnt].Buffer = NULL;
   strcpy(Peer[peer_cnt].Name,"");
}



int mel[256];

void save_file(void)
{
   int fd = open("/tmp/webcam.ppm",O_RDWR|O_CREAT|O_TRUNC|O_NONBLOCK,0644);
   if (fd > 0)
      {
       char *C;
       
       C = (char *)malloc(WIDTH*HEIGHT*3);
       
       sprintf(C,"P6\n%d %d\n%d\n",WIDTH,HEIGHT,255);
       
       write(fd,C,strlen(C));
       
       for (int i=0; i<256; i++)
         mel[i] = 0;
       
       for (int h=0; h<HEIGHT; h++)
          for (int w=0; w<WIDTH; w++)
            {
             int i,j;
             i = w + (h*WIDTH);
             j = (w + (h*WIDTH)) * 3;
            
             mel[picture[i]]++; 
   
             C[j  ] = col[picture[i]].red;
             C[j+1] = col[picture[i]].green;
             C[j+2] = col[picture[i]].blue;
            }


#ifdef STATISTICS       
       int ncols,sumpix;
       ncols = 0;
       sumpix = 0;
       for (int i=0; i<256; i++)
         {
          if (mel[i] > 0)
             {
              fprintf(stdout,"color[%3d]: %6d,   r/g/b:  (0x%02x,0x%02x,0x%02x)\n",
                        i,mel[i],col[i].red,col[i].green,col[i].blue);

              sumpix += mel[i];
              ncols++;
             } 
         }

       fprintf(stdout,"ncolors: %d, pixels: %d\n",ncols,sumpix); 
#endif
              
       write(fd,C,WIDTH*HEIGHT*3);
       
       free(C);
       
       close(fd);
      }    
}

int main(int argc, char *argv[])
{

//fr "select"-aufrufe
//  struct fd_set Write_FD_Set;


  struct sockaddr_in ServerAddr;

//  char buf[BUFSIZE];
//  char st[BUFSIZE];	 	//dummy string
  int I,port,exit_code;
  char message[256];
  
  
  port = 4321;
  
  if (argc>1)
     {
      if (sscanf(argv[1],"%d",&I) == 1)
         port = I;
     } 
   else  
     port = GetTCPPort(WEBCAM_SERV);

  ServerAddr.sin_family = AF_INET;
  ServerAddr.sin_port = htons(port);
  ServerAddr.sin_addr.s_addr=INADDR_ANY;

  Accept_Sock = socket(AF_INET,SOCK_STREAM,0);
  if (Accept_Sock == -1)
    {
     ErrorMsg("socket()",errno);
     exit(2);
    } 

  if (bind(Accept_Sock,(struct sockaddr *)&ServerAddr,sizeof(ServerAddr))==-1)
    {
     ErrorMsg("bind()",errno);
     exit(3); 
    }
  
  if (listen(Accept_Sock,1)==-1)
    {
     ErrorMsg("listen()",errno);
     exit(4);
    }

  fcntl(Accept_Sock,F_SETFL,O_NONBLOCK);
  
  for (int i=0; i<MAX_PEERS; i++)
    {
     Peer[i].Socket = -1;
     Peer[i].connected = false;
     Peer[i].listen = false;
     Peer[i].sent = 0;
     Peer[i].Buffer = NULL;
     strcpy(Peer[i].Name,"");
    } 

  int nc;
  for (int i=0; i<256; i++)
    {
     col[i].red = 0; 
     col[i].green = 0; 
     col[i].blue = 255; 
    }
     
  nc = 16;
  for (int g=0; g<9; g++)
    for (int r=0; r<5; r++)
      for (int b=0; b<5; b++)
        {
         col[nc].green = (g * 255) / 8;
         col[nc].red   = (r * 255) / 4;
         col[nc].blue  = (b * 255) / 4;
        
         nc++; 
        } 


  peer_cnt = 0;				//hlt Anzahl der verbundenen peers (Clienten)
  FD_MaxNum = 0;			//hlt den hchsten Wert eines File-Descriptors
  exit_code = false;  

  init_picture();
  
  while (exit_code == false)
   {  
//    CHECK_POINT("starting while",peer_cnt);
      
    if (peer_cnt < MAX_PEERS)
       accept_new();

//    CHECK_POINT("starting read",1);    
    if (FD_Ready(0) == true)
       {
        char input[128];
        
        int ri = read(0,&input[0],64);
        
        if (ri > 0)
           {
            input[ri] = '\0';
            
            if (strcmp(input,"exit\n") == 0)
               {
                CHECK_POINT("got exit",1);
                exit_code = true;
                break;
               } 
            
            if (strcmp(input,"save\n") == 0)
               {
                CHECK_POINT("got save",1);
                save_file();
                break;
               } 
            
//            if (strcmp(input,"save\n") == 0)
//               save_file();
           }
       } 
    
//    if (i == peer_cnt)
//        usleep(200000);
//     else  
    read_picture();

//    char got_pic = false;
    
    int i;
    i = 0;
   
    while (i<peer_cnt)
      {
       if (process_input(&Peer[i]) == true)
          {
           if (process_output(&Peer[i]) == false)
               delete_peer(&Peer[i]);            
            else  
              i++;
          } 
        else
          delete_peer(&Peer[i]);            
      } // while (i< peer_cnt)
      

//    save_file();  

   } // while not exitcode


  done_picture(); 
  
  for (int i=0; i<peer_cnt; i++)
//    delete_peer(&Peer[i]);
    ShutDown(Peer[i].Socket);
    
  ShutDown(Accept_Sock);
} 











//       CHECK_POINT("reading peer",i);
//            }   
/*

       if (FD_Ready(Peer[i].Socket) == true)
          {
           int rd = read(Peer[i].Socket,&message[0],128);
           
           if (rd < 1)
              {// error
               switch(errno)
                 {
                  case 0: 
                  case 2:
                  case 32:   // brocken pipe
                  case 107:
                    CHECK_POINT("signoff peer",i); 
                    shutdown(Peer[i].Socket,2);
                
                    peer_cnt--;
                    Peer[i].connected = Peer[peer_cnt].connected;
                    Peer[i].Socket = Peer[peer_cnt].Socket;
                    strcpy(Peer[i].Name,Peer[peer_cnt].Name);
  
                    Peer[peer_cnt].Socket = -1;
                    Peer[peer_cnt].connected = false;
                    break;
                    
                  case 11: // EWOULDBLOCK
                    i++;
                    break;
                     
                  default:
                    i++;
                    ErrorMsg("read()",errno);
                    break;    
                 } // switch(errno)
                 
              } // if (rd <= 0)
            else
              {
               int k,wr,r,sz;
               char st[256];
               
               message[rd] = '\0';
               
//               CHECK_POINT(message,rd);
               k = 0;
               while (Cmd[k].ID != 0)
                 {
                  if (strncmp(Cmd[k].str,message,strlen(Cmd[k].str)) == 0)
                     break;
                  k++;
                 }
                 
               wr = 0;  
               switch (Cmd[k].ID)  
                 {
                  case CMD_RESEND:
                    sprintf(st,"%s %%d",Cmd[k].str);
                    
//                    CHECK_POINT(st,k);
                    
                    if (sscanf(message,st,&r) == 1)
                       {
                        sz = PIC_SIZE - r;
                        
                        if ((sz > 0) && (r >= 0)) 
                           {
                            if (sz > FRAG_SIZE)
                               sz = FRAG_SIZE;
                                
                            CHECK_POINT("resend",r);
                            wr = write(Peer[i].Socket,&Peer[i].Buffer[r],sz);
                           } 
                       } 
                    break;
                    
                  case CMD_GET:
//                    CHECK_POINT("writing peer",i);
                    for (int j=0; j<PIC_SIZE; j++)
                       Peer[i].Buffer[j] = picture[j];
                        
                    wr = write(Peer[i].Socket,&Peer[i].Buffer[0],FRAG_SIZE);
       
                    break;  
                    
                  default:
                    CHECK_POINT(message,k);
                    wr = 1;
                    break;  
                 } // switch (cmd code)
              
               
               if (wr > 0)
                   CHECK_POINT("bytes written",wr);
                   
               strcpy(message,"");  

               if (wr == 0)
                  {// error
                   switch(errno)
                     {
                      case 0: 
                      case 2:
                      case 32:   // brocken pipe
                      case 107:
                        shutdown(Peer[i].Socket,2);
                
                        peer_cnt--;
                        Peer[i].connected = Peer[peer_cnt].connected;
                        Peer[i].Socket = Peer[peer_cnt].Socket;
                        strcpy(Peer[i].Name,Peer[peer_cnt].Name);
      
                        Peer[peer_cnt].Socket = -1;
                        Peer[peer_cnt].connected = false;
                        break;  
                
                      default:
                        i++;
                        ErrorMsg("write()",errno);
                        break;    
                     } // switch(errno)
                 
                  } // if (wr == 0)
                  
               } // else if read error
               
          } //if (FD_Ready(Peer[i]) == true)
        else
          i++;   





void save_file(void)
{
   int fd = open("/tmp/webcam.ppm",O_RDWR|O_CREAT|O_TRUNC,0644);
   if (fd > 0)
      {
       char *B ,*C;
       
       C = (char *)malloc(WIDTH*HEIGHT*3*4);
//       B = (char *)malloc(WIDTH*HEIGHT*3*4);
       
       sprintf(C,"P6\n%d %d\n%d\n",WIDTH*2,HEIGHT*2,255);
       
       write(fd,C,strlen(C));
       
       for (int i=0; i<256; i++)
         mel[i] = 0;
       
       for (int h=0; h<HEIGHT; h++)
          for (int w=0; w<WIDTH; w++)
            {
             int i,j;
             i = w + (h*WIDTH);
             j = ((w*2) + (h*2*WIDTH*2)) * 3;
            
             mel[picture[i]]++; 
   
             int idx[3];
             
             idx[0] = 0;
             idx[1] = 1;
             idx[2] = WIDTH*2;
              
             for (int k=0; k<3; k++)            
               {
                int v = (idx[k]*3);
                C[j+0+v] = col[picture[i]].red;
                C[j+1+v] = col[picture[i]].green;
                C[j+2+v] = col[picture[i]].blue;
               } 
            }

       for (int h=1; h<(HEIGHT*2)-1; h++)
          for (int w=1; w<(WIDTH*2)-1; w++)
            {
             unsigned long int S;
             int idx[9];
             int n;
             
             n = 0;
             for (int wi=-1; wi<2; wi++)
               for (int hi=-1; hi<2; hi++)
                 {
                  idx[n] = (w + wi + ((h + hi)*WIDTH*2)) * 3;
                  n++;
                 } 

             for (int k=0; k<3; k++)
               {
                S = 0;
                for (int i=0; i<9; i++)
                   S += C[idx[i]+k];
                   
                S /= 9;    
                B[idx[4]+k] = S;
               }
            } 


       for (int h=1; h<(HEIGHT*2)-1; h++)
          for (int w=1; w<(WIDTH*2)-1; w++)
            {
//             int pre,mid,top,bot,suc,i;   
//             i = w + (h*WIDTH*2);

//             mid = i * 3; 

             if ((h % 2) == 1)
                { // vierfach
                 if ((w % 2) == 1) 
                    { // alle vier

                     unsigned long int S;
                     int idx[9];
                     int n;
             
                     n = 0;
                     for (int wi=-1; wi<2; wi++)
                        for (int hi=-1; hi<2; hi++)
                           {
                            idx[n] = (w + wi + ((h + hi)*WIDTH*2)) * 3;
                            n++;
                           } 

                     for (int k=0; k<3; k++)
                        {
                         S = 0;
                         for (int i=0; i<9; i++)
                           if (i != 4)
                              S += C[idx[i]+k];
                   
                      	 S /= 8;    
                         C[idx[4]+k] = S;
                        }
                    }  
                }
            }

         
       
       int ncols,sumpix;
       ncols = 0;
       sumpix = 0;
       for (int i=0; i<256; i++)
         {
          if (mel[i] > 0)
             {
              fprintf(stdout,"color[%d]: %d,   r/g/b:  (0x%03x,0x%03x,0x%03x)\n",
                        i,mel[i],col[i].red,col[i].green,col[i].blue);

              sumpix += mel[i];
              ncols++;
             } 
         }

       fprintf(stdout,"ncolors: %d, pixels: %d\n",ncols,sumpix); 
       
       write(fd,C,WIDTH*HEIGHT*3*4);
       
//       free(B);
       free(C);
       
       close(fd);
      }    
}

*/

