/* 
 * Thomas Pantzer: xcap_cb.cc, v0.1b  28.1.98 18:6 
 * 
 * 
 * Copyright 1998 by Thomas Pantzer
 * 
 * Permission to use, copy, modify, and distribute this software for noncommercial 
 * use and without fee is hereby granted, provided that the above copyright notice 
 * appear in all copies and that both that copyright notice and this permission 
 * notice appear in supporting documentation, and that the name of the Author not be 
 * used in advertising or publicity pertaining to distribution of the software 
 * without specific, written prior permission. The Author makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is" 
 * without expressed or implied warranty. 
 * 
 * 
 * pantec@aix520.informatik.uni-leipzig.de   (Thomas Pantzer) 
 * 
 */


#include <time.h>
#include <unistd.h>

#include "forms.h"

#include "xcap.h"
#include "capture.h"

/* callbacks for form xcap */


// char win_init = false;


#ifdef DEBUG
#define check_point(x,y)	fprintf(stderr,"%s: p%d\n",x,y)
#else
#define check_point(x,y)	
#endif



void choice_cb(FL_OBJECT *ob, long data)
{
   int i = fl_get_choice(ob);
   
   switch (data)
     {
      case 0:
        if (i > 0)
           xcap_resize(i-1);

        fl_set_choice(fd_xcontrol->vnorm_ch,SizeMode[i-1].pal_mode + 1);   
        break;
        
      case 1:
        vid_norm = i-1;
        xcap_resize(Size);
        break;
     }              
}


void SetBand(long int i)
{
   switch(i)
     {
      case 0:
        fl_set_slider_bounds(fd_xcontrol->freq_sl,48.25,62.25); // 2 .. 4
        fl_set_slider_step(fd_xcontrol->freq_sl,7.0);
        break;
      
      case 1:
        fl_set_slider_bounds(fd_xcontrol->freq_sl,175.25,224.25);  // 5 .. 12
        fl_set_slider_step(fd_xcontrol->freq_sl,7.0);
        break;
        
      case 2:
        fl_set_slider_bounds(fd_xcontrol->freq_sl,471.25,823.25); // 21 .. 69
        fl_set_slider_step(fd_xcontrol->freq_sl,8.0);
        break;
     }        
}

void button_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
  double d,s;
   
  switch(data)
    {
     case -1:
       xcap_exit();
       break;
       
     case 0:
     case 1:
     case 2:
     case 3:
       SetSource((int)data);
       break;    
       
     case 4:
     case 5:
     case 6:
       SetBand(data-4);
       break;  


     case 7: // up
       d = fl_get_slider_value(fd_xcontrol->freq_sl);
       
       if (d > 400.0)
          s = 8.0;
        else
          s = 7.0;  
       
       fl_set_slider_value(fd_xcontrol->freq_sl, d + s);
       
       SetFreq(s + d + fl_get_slider_value(fd_xcontrol->finefrq_sl));
       break;

     case 8: // down
       d = fl_get_slider_value(fd_xcontrol->freq_sl);
       if (d > 400.0)
          s = 8.0;
        else
          s = 7.0;  
       
       fl_set_slider_value(fd_xcontrol->freq_sl, d - s);
       
       SetFreq(- s + d + fl_get_slider_value(fd_xcontrol->finefrq_sl));
       break;
       
    }
}

void slider_cb(FL_OBJECT *ob, long data)
{
  double d;
  
  d = fl_get_slider_value(ob);

  switch(data)
    {
     case 0: // brightness
     case 1: // hue
     case 2: // color
     case 3: // contrast
       SetCtrl(data,d);
       break;      
    
     case 4: // frequency
       SetFreq(d + fl_get_slider_value(fd_xcontrol->finefrq_sl));
       break;
       
     case 5: // adjust
       SetFreq(d + fl_get_slider_value(fd_xcontrol->freq_sl));
       break;  
    }
}

/*
int canvas_expose(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d)
{
//    FD_canvasform *ui = d;
//    XFillRectangle(fl_get_display(), win, canvasGC, 0, 0, w, h);
//    fl_addto_browser(ui->br, "Expose");
    frpintf(stderr,"raise\n");    
    fl_raise_form(fd_xcap->xcap);
    return 0;
}
*/

int canvas_but(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d)
{
//    FD_canvasform *ui = d;
#ifdef DEBUG
    fprintf(stderr,"Button%s: %d\n", ev->type==ButtonPress?"Press":"Release",ev->xbutton.button);
#endif
//    fl_addto_browser(ui->br, buf);

        switch (ev->xbutton.button)
          {
           case 1:  // start/stop grabbing
             break;
             
           case 2:  // switch between bw and colorcube in 8bit mode
             break;  
             
           case 3:  // show/hide control
             if (fd_xcontrol->xcontrol->visible == true)
                fl_hide_form(fd_xcontrol->xcontrol);
              else  
                fl_show_form(fd_xcontrol->xcontrol,FL_PLACE_FREE,FL_FULLBORDER,"xcap control");
             break;  
          }

    return 0;
}



int preemptive_handler(FL_OBJECT *ob, int event,
                          FL_Coord mx, FL_Coord my, int key, void *xev)
{
    XEvent *XEv = (XEvent *)xev;

    check_point("preemptive",1);
 
    switch(event)
    {
      case FL_ENTER:
        fl_raise_form(fd_xcap->xcap);
        break;
/*     
      case FL_LEAVE:
        break;
     
      case FL_PUSH:
        break;
        
      case FL_MOUSE: // one of the quirks of the button class 
        switch (XEv->xbutton.button)
          {
           case 1:  // start/stop grabbing
             break;
             
           case 2:  // switch between bw and colorcube in 8bit mode
             break;  
             
           case 3:  // show/hide control
             if (fd_xcontrol->xcontrol->visible == true)
                fl_hide_form(fd_xcontrol->xcontrol);
              else  
                fl_show_form(fd_xcontrol->xcontrol,FL_PLACE_FREE,FL_FULLBORDER,"xcap control");
             break;  
          }
        break; // FL_MOUSE

      case FL_RELEASE:
        break;
 */        
    }
    return 0;
}


/*
int canvas_key(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d)
{
    FD_canvasform *ui = d;
    char buf[128];
    sprintf(buf, "KeyPress: keysym=%ld", 
           XKeycodeToKeysym(fl_display,ev->xkey.keycode, 0));
    fl_addto_browser(ui->br, buf);
    return 0;
}
*/


time_t stamp_0 = 0;
time_t stamp_1 = 0;

// long idle_cnt = 0;



int nothing_cb(XEvent *,void *)
{
   sleep(20000);
   return 0;
}


int idle_cb(XEvent *,void *)
{
  long i;
  char st[256]; 
  
  stamp_1 = time(0); 

  Do_Capture();
     
  if (stamp_0 == 0)
     { // start-bedingung
      stamp_0 = stamp_1;
      return 0;
     }
  
  i = (stamp_1 - stamp_0);
  
  if (i > 0)
     {
#ifdef DEBUG
      fprintf(stderr,"idle counts = %ld\n",idle_cnt);
#endif      
      idle_cnt = 0;
     } 

  sprintf(st,"%2.1f frames/s",FPS);
  fl_set_object_label(fd_xcontrol->fps_inp,st);  

  seconds += i;
  stamp_0 = stamp_1;
  idle_cnt++;
       
  return 0;
}



